import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Utils";

MouseCircleBounds = class(Turbine.UI.Window);

function MouseCircleBounds:Constructor(parent)
	Turbine.UI.Window.Constructor(self);

	self:SetSize(11, 11);
	self:SetMouseVisible(true);
	self.parent = parent;
	self:SetZOrder(10000);
end

function MouseCircleBounds:MouseEnter(sender, args)
	self.parent:SetVisible(false);
	self.parent.up:SetMouseVisible(false);
	self.parent.left:SetMouseVisible(false);
	self.parent.right:SetMouseVisible(false);
	self.parent.down:SetMouseVisible(false);
end

MouseCircle = class(Turbine.UI.Window);

function MouseCircle:Constructor()
	Turbine.UI.Window.Constructor(self);

	self.settings = MouseCircleLoadSettings();
	local x, y = Turbine.UI.Display:GetMousePosition();
	self.x = x;
	self.y = y;
	self.t = Turbine.Engine.GetGameTime();

	self:SetSize(self.settings.size, self.settings.size);
	self:SetStretchMode(2);
	self:SetBackColor(Turbine.UI.Color(1.0, self.settings.color[0], self.settings.color[1], self.settings.color[2]));
	self:SetBackColorBlendMode(Turbine.UI.BlendMode.Multiply);
	self:SetBackground("moebius92/MouseCircle/circle.tga");
	self:SetVisible(false);
	self:SetMouseVisible(false);
	self:SetWantsUpdates(self.settings.on);
	self:SetZOrder(10000);

	self.up = MouseCircleBounds(self);
	self.down = MouseCircleBounds(self);
	self.left = MouseCircleBounds(self);
	self.right = MouseCircleBounds(self);
end

function MouseCircle:Update()
	local x, y = Turbine.UI.Display:GetMousePosition();
	local t = Turbine.Engine.GetGameTime();

	if self.x == x and self.y == y then
		if (t - self.settings.delay) > self.t then
			self:SetVisible(true);
			self:SetPosition(x - self.settings.size / 2, y - self.settings.size / 2);
			self.up:SetPosition(x - 5, y - 12);
			self.left:SetPosition(x - 12, y - 5);
			self.right:SetPosition(x + 1, y - 5);
			self.down:SetPosition(x - 5, y + 1);
			self.up:SetMouseVisible(true);
			self.left:SetMouseVisible(true);
			self.right:SetMouseVisible(true);
			self.down:SetMouseVisible(true);
		end
	else
		self.x = x;
		self.y = y;
		self.t = t;
		self:SetVisible(false);
		self.up:SetMouseVisible(false);
		self.left:SetMouseVisible(false);
		self.right:SetMouseVisible(false);
		self.down:SetMouseVisible(false);
	end
end

function MouseCircle:SaveSettings()
	Turbine.PluginData.Save(Turbine.DataScope.Account, "MouseCircle", self.settings);
end

function MouseCircleLoadSettings()
	settings = Turbine.PluginData.Load(Turbine.DataScope.Account, "MouseCircle");
	if type(settings) ~= "table" then
		settings = { };
	end

	if not settings.size then
		settings.size = 50;
	end
	if not settings.on then
		settings.on = true;
	end
	if not settings.delay then
		settings.delay = 0.5;
	end
	if not settings.color then
		settings.color = { };
		settings.color[0] = 1.0;
		settings.color[1] = 0.5;
		settings.color[2] = 0.5;
	end

	return settings;
end

MouseCircleCommand = class(Turbine.ShellCommand);

function MouseCircleCommand:Execute(command, arguments)
	local i = string.find(arguments, " ");
	local arg, rest;

	if i ~= nil then
		arg = string.sub(arguments, 0, i - 1);
		rest = string.sub(arguments, i + 1, -1);
	else
		arg = arguments;
		rest = nil;
	end

	if arg == "color" then
		local r, g, b;

		i = string.find(rest, " ");
		arg = string.sub(rest, 0, i - 1);
		rest = string.sub(rest, i + 1, -1);
		r = tonumber(arg);
		i = string.find(rest, " ");
		arg = string.sub(rest, 0, i - 1);
		rest = string.sub(rest, i + 1, -1);
		g = tonumber(arg);
		b = tonumber(rest);

		if r ~= nil and g ~= nil and b ~= nil then
			mouseCircle.settings.color[0] = r;
			mouseCircle.settings.color[1] = g;
			mouseCircle.settings.color[2] = b;
			mouseCircle:SetBackColor(Turbine.UI.Color(1.0, r, g, b));
			mouseCircle:SaveSettings();
		end

		Turbine.Shell.WriteLine("MouseCircle color is (" .. mouseCircle.settings.color[0] .. "," .. mouseCircle.settings.color[1] .. "," .. mouseCircle.settings.color[2] .. ")");
	elseif arg == "toggle" then
		if mouseCircle.settings.on then
			mouseCircle.settings.on = false;
			mouseCircle:SetWantsUpdates(false);
			mouseCircle:SetVisible(false);
			Turbine.Shell.WriteLine("MouseCircle is off.");
		else
			mouseCircle.settings.on = true;
			mouseCircle:SetWantsUpdates(true);
			Turbine.Shell.WriteLine("MouseCircle is on.");
		end
		mouseCircle:SaveSettings();
	elseif arg == "on" then
		mouseCircle.settings.on = true;
		mouseCircle:SetWantsUpdates(true);
		Turbine.Shell.WriteLine("MouseCircle is on.");
		mouseCircle:SaveSettings();
	elseif arg == "off" then
		mouseCircle.settings.on = false;
		mouseCircle:SetWantsUpdates(false);
		mouseCircle:SetVisible(false);
		Turbine.Shell.WriteLine("MouseCircle is off.");
		mouseCircle:SaveSettings();
	elseif arg == "delay" then
		local x = tonumber(rest);
		if x ~= nil then
			mouseCircle.settings.delay = rest;
			mouseCircle:SaveSettings();
		end
		Turbine.Shell.WriteLine("MouseCircle delay is " .. mouseCircle.settings.delay .. " seconds.");
	elseif arg == "size" then
		local x = tonumber(rest);
		if x ~= nil then
			mouseCircle.settings.size = x;
			mouseCircle:SaveSettings();
			mouseCircle:SetSize(x, x);
		end
		Turbine.Shell.WriteLine("MouseCircle is " .. mouseCircle.settings.size .. "x" .. mouseCircle.settings.size .. ".");
	else
		Turbine.Shell.WriteLine("usage: /mousecircle <color <red> <green> <blue>|toggle|on|off|delay <sec>|size <pixels>>");
	end
end

function MouseCircleCommand:GetShortHelp()
	return "usage: /mousecircle <color|toggle|on|off|delay|size>";
end

mouseCircleCommand = MouseCircleCommand();
Turbine.Shell.AddCommand('mousecircle', mouseCircleCommand);
Turbine.Shell.AddCommand('mc', mouseCircleCommand);
mouseCircle = MouseCircle();

if mouseCircle.settings.on then
	Turbine.Shell.WriteLine("MouseCircle is on.");
else
	Turbine.Shell.WriteLine("MouseCircle is off.");
end